package gregapi.block;

import static gregapi.data.CS.*;
import gregapi.code.ItemStackContainer;
import gregapi.data.CS;
import gregapi.data.CS.ModIDs;
import gregapi.data.TD;
import gregapi.oredict.OreDictItemData;
import gregapi.recipes.Recipe;
import gregapi.recipes.Recipe.RecipeMap;
import gregapi.util.OM;
import gregapi.util.UT;
import ic2.api.crops.ICropTile;
import ic2.api.tile.IWrenchable;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;

import net.minecraft.block.*;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.common.util.ForgeDirection;
import net.minecraftforge.event.entity.player.UseHoeEvent;
import net.minecraftforge.fluids.IFluidBlock;
import cpw.mods.fml.common.FMLLog;
import cpw.mods.fml.common.Loader;

/**
 * @author Gregorius Techneticies
 * 
 * For Internal Use.
 */
public class ToolCompat {
	private static boolean IC_WRENCHABLE = false, IC_CROPTILE = false, RC_INSTALLED = false, CHECK_ALL = true;
	
	private static void checkAvailabilities() {
		if (CHECK_ALL) {
			try {
				IWrenchable.class.getCanonicalName();
				IC_WRENCHABLE = true;
			} catch(Throwable e) {/**/}
			try {
				ICropTile.class.getCanonicalName();
				IC_CROPTILE = true;
			} catch(Throwable e) {/**/}
			RC_INSTALLED = Loader.isModLoaded(ModIDs.RC);
			CHECK_ALL = false;
		}
	}
	
	/** Providing compatibility for vanilla Blocks and certain Mod Interfaces. */
	public static long onToolClick(Block aBlock, String aTool, long aRemainingDurability, long aQuality, Entity aPlayer, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, World aWorld, byte aSide, int aX, int aY, int aZ, float aHitX, float aHitY, float aHitZ) {
		checkAvailabilities();
		
		byte aMeta = (byte)aWorld.getBlockMetadata(aX, aY, aZ);
		TileEntity aTileEntity = UT.Worlds.getTileEntity(aWorld, aX, aY, aZ, true);
		EntityPlayer aEntityPlayer = aPlayer instanceof EntityPlayer ? (EntityPlayer)aPlayer : null;
		EntityLivingBase aEntityLiving = aPlayer instanceof EntityLivingBase ? (EntityLivingBase)aPlayer : null;
		
		try {
		
		if (aTool.equals(TOOL_hoe) && (aEntityPlayer == null || aEntityPlayer.canPlayerEdit(aX, aY, aZ, aSide, aStack))) {
			if (!MinecraftForge.EVENT_BUS.post(new UseHoeEvent(aEntityPlayer, aStack, aWorld, aX, aY, aZ))) {
				if (SIDES_TOP_HORIZONTAL[aSide] && !UT.Worlds.hasBlockHitBox(aWorld, aX, aY+1, aZ) && (aBlock == Blocks.grass || aBlock == Blocks.dirt)) {
					aWorld.playSoundEffect(aX + 0.5F, aY + 0.5F, aZ + 0.5F, Blocks.farmland.stepSound.getStepResourcePath(), (Blocks.farmland.stepSound.getVolume() + 1.0F) / 2.0F, Blocks.farmland.stepSound.getPitch() * 0.8F);
				    if (!aWorld.isRemote) aWorld.setBlock(aX, aY, aZ, Blocks.farmland);
					return 10000;
				}
			}
		}
		
		
		if (aWorld.isRemote) return 0;
		
		
		if (aTool.equals(TOOL_sense) || aTool.equals(TOOL_scythe)) {
	    	if (IC_CROPTILE && aTileEntity instanceof ICropTile) {
	    		int tDamage = 0;
	    		for (int i = -1; i < 2; i++) for (int j = -1; j < 2; j++) for (int k = -1; k < 2; k++) if ((aTileEntity = UT.Worlds.getTileEntity(aWorld, aX+i, aY+j, aZ+k, true)) instanceof ICropTile && ((ICropTile)aTileEntity).harvest(true)) tDamage += 10000;
	    		return tDamage;
	    	}
		}
		if (aTool.equals(TOOL_igniter) && (aStack == null || aStack.getItem() != Items.flint_and_steel)) {
			if (aBlock instanceof BlockTNT) {
				((BlockTNT)aBlock).func_150114_a(aWorld, aX, aY, aZ, aMeta, aEntityLiving);
				return 10000;
			}
			if (aEntityPlayer == null || aEntityPlayer.canPlayerEdit(aX+OFFSETS_X[aSide], aY+OFFSETS_Y[aSide], aZ+OFFSETS_Z[aSide], aSide, aStack)) {
	            if (aWorld.isAirBlock(aX+OFFSETS_X[aSide], aY+OFFSETS_Y[aSide], aZ+OFFSETS_Z[aSide])) {
	                aWorld.setBlock(aX+OFFSETS_X[aSide], aY+OFFSETS_Y[aSide], aZ+OFFSETS_Z[aSide], Blocks.fire);
		            return 10000;
	            }
			}
		}
		if (aTool.equals(TOOL_chisel)) {
			ItemStack tChiseledBlock = UT.Worlds.getStack(aWorld, aX, aY, aZ);
			if (tChiseledBlock != null) {
				Recipe tRecipe = RecipeMap.sChiselRecipes.findRecipe(null, null, true, Integer.MAX_VALUE, null, ZL_FLUIDSTACK, tChiseledBlock);
				if (tRecipe != null
				&& tRecipe.mInputs .length == 1 && tRecipe.mInputs [0] != null && tRecipe.mInputs [0].stackSize == 1 && UT.Stacks.equal(tRecipe.mInputs[0], tChiseledBlock)
				&& tRecipe.mOutputs.length == 1 && tRecipe.mOutputs[0] != null && tRecipe.mOutputs[0].stackSize == 1 && UT.Worlds.setBlock(aWorld, aX, aY, aZ, tRecipe.mOutputs[0])
				) return 10000;
			}
		}
		if (aTool.equals(TOOL_rotator)) {
			if (aBlock.rotateBlock(aWorld, aX, aX, aX, ForgeDirection.getOrientation(aSide))) return 10000;
		}
		if (aTool.equals(TOOL_screwdriver)) {
	    	if (aBlock instanceof BlockRedstoneDiode) {
				if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, (aMeta / 4) * 4  + (((aMeta%4) + 1) % 4), 3)) return 10000;
	    	}
		}
		if (aTool.equals(TOOL_crowbar)) {
			if (!RC_INSTALLED && aBlock instanceof BlockRailBase) {
				aWorld.isRemote = true;
				// WTF, why are the two Coordinate Parameters in isFlexibleRail switched? And then it is used like x y z instead of using the broken namings.
				boolean tResult = aWorld.setBlock(aX, aY, aZ, aBlock, ((BlockRailBase)aBlock).isFlexibleRail(aWorld, aX, aY, aZ)?(aMeta + 1) % 10:((aMeta / 8) * 8) + (((aMeta%8)+1) % 6), 0);
				aWorld.isRemote = false;
		    	return tResult?10000:0;
			}
		}
		if (aTool.equals(TOOL_softhammer)) {
	    	if (aBlock == Blocks.lit_redstone_lamp) {
				aWorld.isRemote = true;
				boolean tResult = aWorld.setBlock(aX, aY, aZ, Blocks.redstone_lamp, 0, 0);
				aWorld.isRemote = false;
	    		return tResult?10000:0;
	    	}
	    	if (aBlock == Blocks.redstone_lamp) {
				aWorld.isRemote = true;
				boolean tResult = aWorld.setBlock(aX, aY, aZ, Blocks.lit_redstone_lamp, 0, 0);
				aWorld.isRemote = false;
	    		return tResult?10000:0;
	    	}
	    	if (aBlock == Blocks.golden_rail) {
				aWorld.isRemote = true;
				boolean tResult = aWorld.setBlock(aX, aY, aZ, aBlock, (aMeta + 8) % 16, 0);
				aWorld.isRemote = false;
	    		return tResult?10000:0;
	    	}
	    	if (aBlock == Blocks.activator_rail) {
				aWorld.isRemote = true;
				boolean tResult = aWorld.setBlock(aX, aY, aZ, aBlock, (aMeta + 8) % 16, 0);
				aWorld.isRemote = false;
	    		return tResult?10000:0;
	    	}
	    	if (aBlock instanceof BlockRotatedPillar) {
				if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, (aMeta + 4) % 12, 3)) return 10000;
	    	}
		    if (aBlock == Blocks.piston || aBlock == Blocks.sticky_piston || aBlock == Blocks.dispenser || aBlock == Blocks.dropper) {
		    	if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, (aMeta+1) % 6, 3)) return 10000;
		    }
		    if (aBlock == Blocks.pumpkin || aBlock == Blocks.lit_pumpkin || aBlock == Blocks.furnace || aBlock == Blocks.lit_furnace || aBlock == Blocks.chest || aBlock == Blocks.trapped_chest) {
		    	if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, ((aMeta-1)%4)+2, 3)) return 10000;
		    }
		    if (aBlock == Blocks.hopper) {
		    	if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, (aMeta+1)%6==1?(aMeta+1)%6:2, 3)) return 10000;
		    }
		}
		if (aTool.equals(TOOL_prospector)) {
	    	OreDictItemData tAssotiation = OM.association((aBlock instanceof IBlockRetrievable ? ((IBlockRetrievable)aBlock).getItemStackFromBlock(aWorld, aX, aY, aZ, SIDE_INVALID) : new ItemStack(aBlock, 1, aMeta)));
	    	if (tAssotiation != null && tAssotiation.mPrefix.contains(TD.Prefix.ORE)) {
				UT.Entities.chat(aPlayer, "This is " + tAssotiation.mMaterial.mMaterial.getLocal() + " Ore.");
	    		return 1000;
	    	}
	    	
		    if (aBlock.isReplaceableOreGen(aWorld, aX, aY, aZ, Blocks.stone) || aBlock.isReplaceableOreGen(aWorld, aX, aY, aZ, Blocks.netherrack) || aBlock.isReplaceableOreGen(aWorld, aX, aY, aZ, Blocks.end_stone) || CS.BlocksGT.stoneToBrokenOres.containsKey(new ItemStackContainer(aBlock, 1, aMeta)) || CS.BlocksGT.stoneToNormalOres.containsKey(new ItemStackContainer(aBlock, 1, aMeta)) || CS.BlocksGT.stoneToSmallOres.containsKey(new ItemStackContainer(aBlock, 1, aMeta))) {
				int tX = aX, tY = aY, tZ = aZ;
				Block tBlock;
		        for (int i = 0, j = (int)(6 + aQuality); i < j; i++) {
		        	tX -= OFFSETS_X[aSide];
		        	tY -= OFFSETS_Y[aSide];
		        	tZ -= OFFSETS_Z[aSide];
		        	
					tBlock = aWorld.getBlock(tX, tY, tZ);
			    	if (tBlock == Blocks.lava || tBlock == Blocks.flowing_lava) {
			    		UT.Entities.chat(aPlayer, "There is Lava behind this Rock.");
				    	break;
			    	}
			    	if (tBlock == Blocks.water || tBlock == Blocks.flowing_water || tBlock instanceof IFluidBlock) {
			    		UT.Entities.chat(aPlayer, "There is a Liquid behind this Rock.");
				    	break;
					}
			    	if (tBlock == Blocks.monster_egg || !UT.Worlds.hasBlockHitBox(aWorld, tX, tY, tZ)) {
			    		UT.Entities.chat(aPlayer, "There is an Air Pocket behind this Rock.");
				    	break;
			    	}
			    	if (tBlock != aBlock) {
			    		if (i < 4) UT.Entities.chat(aPlayer, "Material is changing behind this Rock.");
				    	break;
			    	}
			    }
		        
				Random tRandom = new Random(aX^aY^aZ^aSide);
				for (int i = 0, j = (int)(9+2*aQuality); i < j; i++) {
					tX = (int) (aX-4-aQuality+tRandom.nextInt(j));
					tY = (int) (aY-4-aQuality+tRandom.nextInt(j));
					tZ = (int) (aZ-4-aQuality+tRandom.nextInt(j));
					tBlock = aWorld.getBlock(tX, tY, tZ);
					
					tAssotiation = OM.association((tBlock instanceof IBlockRetrievable ? ((IBlockRetrievable)tBlock).getItemStackFromBlock(aWorld, tX, tY, tZ, SIDE_INVALID) : new ItemStack(tBlock, 1, aWorld.getBlockMetadata(tX, tY, tZ))));
					if (tAssotiation != null && tAssotiation.mPrefix.contains(TD.Prefix.ORE)) {
					    UT.Entities.chat(aPlayer, "Found traces of " + tAssotiation.mMaterial.mMaterial.getLocal() + " Ore.");
						return 10000;
					}
				}
				UT.Entities.chat(aPlayer, "No Ores found.");
	    		return 10000;
		    }
		}
		
		if (aTool.equals(TOOL_wrench) || aTool.equals(TOOL_monkeywrench)) {
			byte aTargetSide = UT.Code.getSideWrenching(aSide, aHitX, aHitY, aHitZ);
	        if (IC_WRENCHABLE && aTileEntity instanceof IWrenchable) {
	       		if (((IWrenchable)aTileEntity).wrenchCanSetFacing(aEntityPlayer, aTargetSide)) {
	           		((IWrenchable)aTileEntity).setFacing(aTargetSide);
	       			return 10000;
	       		}
	        	if (((IWrenchable)aTileEntity).wrenchCanRemove(aEntityPlayer)) {
	        		int tDamage = Math.max(10000, (int)(30000 / ((IWrenchable)aTileEntity).getWrenchDropRate()));
	        		ArrayList<ItemStack> tDrops = aBlock.getDrops(aWorld, aX, aY, aZ, aMeta, 0);
	        		ItemStack tOutput = ((IWrenchable)aTileEntity).getWrenchDrop(aEntityPlayer);
	        		
	        		if (aWorld.setBlockToAir(aX, aY, aZ)) {
		        		if (aWorld.rand.nextInt(tDamage) < aRemainingDurability) {
			            	for (ItemStack tStack : tDrops) {
			            		if (tOutput == null) {
			                		aWorld.spawnEntityInWorld(new EntityItem(aWorld, aX+0.5, aY+0.5, aZ+0.5, tStack));
			            		} else {
			                		aWorld.spawnEntityInWorld(new EntityItem(aWorld, aX+0.5, aY+0.5, aZ+0.5, tOutput));
			                		tOutput = null;
			            		}
			            	}
		        		}
		        		return tDamage;
	        		}
	        	}
	        }
	    	
	    	if (aBlock instanceof BlockRotatedPillar) {
				if (aWorld.setBlockMetadataWithNotify(aX, aY, aZ, (aMeta + 4) % 12, 3)) return 10000;
	    	}
	    	
	    	if (aBlock instanceof BlockWorkbench || aBlock instanceof BlockBookshelf) {
	    		if (aWorld.setBlockToAir(aX, aY, aZ)) {
        			aWorld.spawnEntityInWorld(new EntityItem(aWorld, aX+0.5, aY+0.5, aZ+0.5, new ItemStack(aBlock, 1, aMeta)));
		    		return 10000;
        		}
	    	}
	    	
	    	if (aMeta == aTargetSide) {
		    	if (aBlock instanceof BlockPumpkin || aBlock instanceof BlockPistonBase || aBlock instanceof BlockDispenser || aBlock instanceof BlockFurnace || aBlock instanceof BlockChest || aBlock instanceof BlockHopper || aBlock instanceof BlockEnderChest) {
	        		if (aWorld.setBlockToAir(aX, aY, aZ)) {
	        			aWorld.spawnEntityInWorld(new EntityItem(aWorld, aX+0.5, aY+0.5, aZ+0.5, new ItemStack(aBlock, 1, 0)));
			    		return 10000;
	        		}
		    	}
	    	} else {
		    	if (aBlock instanceof BlockPistonBase || aBlock instanceof BlockDispenser) {
					if (aMeta < 6 && aWorld.setBlockMetadataWithNotify(aX, aY, aZ, aTargetSide, 3)) return 10000;
		    	}
		    	if (aBlock instanceof BlockPumpkin || aBlock instanceof BlockFurnace || aBlock instanceof BlockChest || aBlock instanceof BlockEnderChest) {
					if (aTargetSide > 1 && aWorld.setBlockMetadataWithNotify(aX, aY, aZ, aTargetSide, 3)) return 10000;
		    	}
		    	if (aBlock instanceof BlockHopper) {
					if (aTargetSide != 1 && aWorld.setBlockMetadataWithNotify(aX, aY, aZ, aTargetSide, 3)) return 10000;
		    	}
	    	}
	    	if (Arrays.asList(aBlock.getValidRotations(aWorld, aX, aY, aZ)).contains(ForgeDirection.getOrientation(aTargetSide))) {
	    		if (aBlock.rotateBlock(aWorld, aX, aY, aZ, ForgeDirection.getOrientation(aTargetSide))) return 10000;
	    	}
		}
		
		} catch(Throwable e) {
			FMLLog.severe("Exception occured when ToolCompat was used at the Coordinates: [%d;%d;%d] at '%s' with TileEntity '%s' using the Tool '%s' %s", aX, aY, aZ, aBlock.getUnlocalizedName(), aTileEntity.getClass(), aTool, e.toString());
		}
		return 0;
	}
}