package gregapi.config;

import gregapi.GT_API;
import gregapi.oredict.OreDictManager;
import gregapi.util.UT;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.config.Configuration;
import net.minecraftforge.common.config.Property;

/**
 * @author Gregorius Techneticies
 */
public class Config implements Runnable {
	public static Configuration sConfigFileIDs;
	private static boolean sIDConfigNeedsSaving;
	
	public static int addIDConfig(Object aCategory, String aName, int aDefault) {
		if (UT.Code.stringInvalid(aName)) return aDefault;
		Property tProperty = sConfigFileIDs.get(aCategory.toString().replaceAll("\\|", "."), aName.replaceAll("\\|", "."), aDefault);
		int rResult = tProperty.getInt(aDefault);
		if (!tProperty.wasRead()) if (GT_API.sEnableDynamicWritingToConfig) sConfigFileIDs.save(); else sIDConfigNeedsSaving = true;
		return rResult;
	}
	
	public final Configuration mConfig;
	
	public Config(Configuration aConfig) {
		mConfig = aConfig;
		mConfig.load();
		mConfig.save();
		if (GT_API.sAPIPreInitAfter	!= null)	GT_API.sAPIPreInitAfter			.add(this); // in case of crash on startup
		if (GT_API.sGTPreInitAfter	!= null)	GT_API.sGTPreInitAfter			.add(this); // in case of crash on startup
		/* No Null-Check necessary because */	GT_API.sAPIInitAfter			.add(this); // in case of crash on startup
		/* Configurations go into preInit. */	GT_API.sGTInitAfter				.add(this); // in case of crash on startup
												GT_API.sAPIPostInitAfter		.add(this); // in case of the Player not starting a Game
												GT_API.sGTPostInitAfter			.add(this); // in case of the Player not starting a Game
	}
	
	@Override
	public void run() {
		mConfig.save();
		if (sIDConfigNeedsSaving) {
			sConfigFileIDs.save();
			sIDConfigNeedsSaving = false;
		}
	}
	
	public static String getStackConfigName(ItemStack aStack) {
		if (UT.Stacks.invalid(aStack)) return "";
		Object rName = OreDictManager.INSTANCE.getAssociation(aStack);
		if (rName != null) return rName.toString();
		try {if (UT.Code.stringValid(rName = aStack.getUnlocalizedName())) return rName.toString();} catch (Throwable e) {/*Do nothing*/}
		return aStack.getItem() + "." + aStack.getItemDamage();
	}
	
	public boolean get(Object aCategory, ItemStack aStack, boolean aDefault) {
		return get(aCategory, getStackConfigName(aStack), aDefault);
	}
	
	public boolean get(Object aCategory, String aName, boolean aDefault) {
		if (UT.Code.stringInvalid(aName)) return aDefault;
		Property tProperty = mConfig.get(aCategory.toString().replaceAll("\\|", "_"), (aName+"_"+aDefault).replaceAll("\\|", "_"), aDefault);
		boolean rResult = tProperty.getBoolean(aDefault);
		if (GT_API.sEnableDynamicWritingToConfig && !tProperty.wasRead()) mConfig.save();
		return rResult;
	}
	
	@Deprecated
	public int get(Object aCategory, ItemStack aStack, int aDefault) {
		return get(aCategory, getStackConfigName(aStack), (long)aDefault);
	}
	
	@Deprecated
	public int get(Object aCategory, String aName, int aDefault) {
		return get(aCategory, aName, (long)aDefault);
	}
	
	public int get(Object aCategory, ItemStack aStack, long aDefault) {
		return get(aCategory, getStackConfigName(aStack), aDefault);
	}
	
	public int get(Object aCategory, String aName, long aDefault) {
		if (UT.Code.stringInvalid(aName)) return UT.Code.bindInt(aDefault);
		Property tProperty = mConfig.get(aCategory.toString().replaceAll("\\|", "_"), (aName+"_"+UT.Code.bindInt(aDefault)).replaceAll("\\|", "_"), UT.Code.bindInt(aDefault));
		int rResult = tProperty.getInt(UT.Code.bindInt(aDefault));
		if (GT_API.sEnableDynamicWritingToConfig && !tProperty.wasRead()) mConfig.save();
		return rResult;
	}
	
	public String get(Object aCategory, ItemStack aStack, String aDefault) {
		return get(aCategory, getStackConfigName(aStack), aDefault);
	}
	
	public String get(Object aCategory, String aName, String aDefault) {
		if (UT.Code.stringInvalid(aName)) return aDefault;
		Property tProperty = mConfig.get(aCategory.toString().replaceAll("\\|", "_"), (aName+"_"+aDefault).replaceAll("\\|", "_"), aDefault);
		String rResult = tProperty.getString();
		if (GT_API.sEnableDynamicWritingToConfig && !tProperty.wasRead()) mConfig.save();
		return rResult;
	}
	
	public double get(Object aCategory, ItemStack aStack, double aDefault) {
		return get(aCategory, getStackConfigName(aStack), aDefault);
	}
	
	public double get(Object aCategory, String aName, double aDefault) {
		if (UT.Code.stringInvalid(aName)) return aDefault;
		Property tProperty = mConfig.get(aCategory.toString().replaceAll("\\|", "_"), (aName+"_"+aDefault).replaceAll("\\|", "_"), aDefault);
		double rResult = tProperty.getDouble(aDefault);
		if (GT_API.sEnableDynamicWritingToConfig && !tProperty.wasRead()) mConfig.save();
		return rResult;
	}
}